## Copyright (C) 2011 Pantxo Diribarne
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} {} pgf_write_table (@var{filepath}, @var{varname}, @var{varvalues}, ...)
## 
##
## Writes a data file for use with the pgfplots command '\addplot table':
##
## - @var{filepath} : relative path to the file. Warning : if the file
## already exists, its content is overwritten.
##
## - @var{varname}, @var{varvalue} : the function can treat an arbitrary
## number of pairs @var{varname}, @var{varvalue}. The @var{varname}
## parameter is expected to be a string which will be the written in
## the header of each data column, and will thus designate the dataset
## in pgfplots. The parameter @var{varvalue} is expected to be a one
## dimensionnal numerical values array.
## If the datasets don't have the same size, the order of the data
## columns is modified so that it remains man readable (sorted by decreasing
## length).
##
## Example :
## @example
## @group
## pgf_write_table ('./Dataset.table', 'X', (1:10), 'SquareX', (1:10).^2);
## @end group
## @end example
## @end deftypefn

## Author: Pantxo Diribarne <pantxodiribarne@gamb-port-dp.ujf-grenoble.fr>
## Created: 2011-08-31

function pgf_write_table (filename , varargin)
  more off
  if iscell(varargin{1})
    varargin = varargin{1};
  endif
  if isodd (numel (varargin))
    print_usage ("to_pgf_table : wrong arg number")
  else
    var = varargin(1:2:end);
    val = varargin(2:2:end);
    ncol = numel (var);
    
    ## count and reshape data to col vector 
    max_lg = 0;
    temp = [];
    for ii = 1:ncol
      if isvector (val {ii})
        n = length (val{ii});
        temp = [temp ; n];
        if rows (val{ii}) == 1
          val{ii} = val{ii}';
        endif
      else
        error ("to_pgf_table : wrong type of argument n#%d", ii+1)
      endif
    endfor
    [temp, idx] = sort(temp, "descend");
    var = var(idx);
    val = val(idx);
    nrow = temp(1);
    
    ## write header
    id = fopen (filename, "w");
    for ii=1:ncol
      fprintf (id, "%s ", var{ii});
    endfor
    fprintf (id, "\n");
    fclose (id);
    
    ## write data
    if any(find (diff (temp)))
      test = -1*diff (temp);
      n = ncol;
      start = 1;
      stop = temp(n);
      while n > 1
        tab = [];

        for ii=1:n
          tab = [tab val{ii}(start:stop)]; 
        endfor
        save("-ascii", "-append", filename, "tab");
        n--
        
        if stop < max (temp);
          if (n == 1)
            warning ("to_pgf_table: non-conformant vector dimensions")
          elseif test(n)
            start = stop+1;
            stop = start + test(n) - 1;
          else
            while !test(n) 
              n--;
            endwhile
            start = stop+1;
            stop = start + test(n) - 1;
          endif
        else
          n = 1;
        endif
      endwhile
    else
      tab =[];
      for ii=1:ncol
        tab = [tab val{ii}];
      endfor
      save("-ascii", "-append", filename, "tab");
    endif
    
  endif
endfunction

function out = isodd (eltnum)
  if (floor (eltnum/2) == (eltnum-1))
    out = 1;
  else
    out = 0;
  endif
endfunction
