## Copyright (C) 2013 Pantxo Diribarne
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} {} pgfcompile (@var{filepath})
## @deftypefnx {Function File} {} pgfcompile (@var{filepath}, @var{form})
## @deftypefnx {Function File} {} pgfcompile (@var{filepath}, @var{form}, @var{cmd})
## @deftypefnx {Function File} {} pgfcompile (@var{filepath}, @var{form}, @var{cmd}, @var{keepmaster})
## Compile the file designated by @var{filepath} (without .tex extension)
## and output a standalone image. The format of the image @var{form}
## may be "pdf"(default) or "eps". The command @var{cmd} to be used
## may be "pdflatex" or "lualatex" (default). If @var{keepmaster}
## argument is set to true, the master tex file used to compile the
## figure is conserved (may be useful for debuging).
## @seealso{pgf_write_table, pgfit}
## @end deftypefn

## Author: Pantxo Diribarne <pantxo@dibona>
## Created: 2013-08-27

function pgfcompile (filepath, form = "pdf", ...
                     cmd = "lualatex", keepmaster = false)

  %Check the platform type
  iswin = strcmp (filesep, "\\");

  
  if (iswin)
    [tmp1, s1] = system (sprintf ("where %s", cmd));
  else
    [tmp1, s1] = system (sprintf ("which %s", cmd));
  endif
  if (tmp1 != 0)
    error ("pgfcompile: could not find %s", cmd)
  endif

  if (strcmp (form, "eps"))
    cmd = "lualatex";
    if (iswin)
      [tmp1, s1] = system ("where ps2eps");
    else
      [tmp1, s1] = system ("which ps2eps");
    endif
    if (tmp1 != 0)
      warning ("pgfcompile: didn't find ps2eps. Producing pdf output!")
      form = "pdf";
    endif
  endif
  [master] = pgf_write_masterfile (filepath, form);

  
  unwind_protect
    system (sprintf ("%s -shell-escape %s.tex", cmd, master));
    if (form == "pdf")
      rename ([master, ".pdf"],
              [filepath, ".pdf"]);
    elseif (form == "eps")
      system (sprintf ("pdf2ps %s.pdf", master));
      system (sprintf ("ps2eps -l %s.ps", master));
      rename ([master, ".eps"],
              [filepath, ".eps"]);
    endif
  unwind_protect_cleanup
    if (!keepmaster)
      system (sprintf ("rm -f %s.*", master));
    endif
  end_unwind_protect
endfunction


function [master] = pgf_write_masterfile (file_path, form)
  
  [dr, master] = fileparts (tmpnam ());
  [extern_id, msg] = fopen ([master ".tex"], "w+");
  ## [dr, fig] = fileparts (file_path);
  ## fig = [fig "_fig"];
  if (extern_id == -1)
    error ("pgfit: couldn't open %s,\n%s",
           fullfile (dr, [master ".tex"]), disp (msg))
  endif
  ##shell escape
  shellesc = "\\tikzset{external/system call={%s \\tikzexternalcheckshellescape -halt-on-error -interaction=batchmode -jobname \"\\image\" \"\\texsource\"%s}}\n";
  if (strcmp (form, "eps"))
    cmd = "pdflatex";
    sufix = " && pdf2ps \"\\image\".pdf";
    sufix = [sufix " && ps2eps -l \"\\image\".ps"];
  else
    cmd = "pdflatex";
    sufix = "";
  endif
  shellesc = sprintf (shellesc, cmd, sufix);
  
  str = pgf_header_str ();
  fprintf (extern_id, str);
  fprintf (extern_id, "\n");
  ## fprintf (extern_id, "\\usepgfplotslibrary{external} \n");
  ## fprintf (extern_id, shellesc);
  ## fprintf (extern_id, "\\tikzexternalize\n");
  fprintf (extern_id, "\\begin{document}\n");
  ## fprintf (extern_id, "\\tikzsetnextfilename{%s} \n", fig);
  fprintf (extern_id, "\\input{%s}\n", file_path);
  fprintf (extern_id, "\\end{document} \n");
  fclose (extern_id);
endfunction

function [ str ] = pgf_header_str ()
  
  str = sprintf ("\\documentclass[tikz]{standalone}\n");
  ## str = sprintf ("%s\\usepackage{etex} \n", str);
  str = sprintf ("%s\\usepackage{pgfplots} \n", str);
  str = sprintf ("%s\\pgfplotsset{compat=newest} \n", str);
  ## str = sprintf ("%s\\usepackage[T1]{fontenc} \n", str);
  str = sprintf ("%s\\usepackage{lmodern} \n", str);
  str = sprintf ("%s\\usepackage{amsmath, amssymb} \n", str);
  str = sprintf ("%s\\usetikzlibrary{plotmarks, shapes.multipart} \n", str);
  
endfunction
