## Copyright (C) 2012 pdiribarne
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn  {Function File} {} pgfit (@var{H}, @var{filepath})
## @deftypefnx {Function File} {} pgfit (@var{H}, @var{filepath}, @var{opt}, @var{value}, ...)
## 
## Exports a figure, associated to handle @var{H}, to a pgplot tex
## file with separate data files.
##
## Supported features are: lines, errorbars, contours, surfaces,
## images (not for "eps" output), plotyy and multiple axes.
## @var{filepath} is the file path
## where the tikzpicture will be written. If the picture is to be used
## in a different directory than its master file, use relative path
## e.g.:
## @example
## @group
## pgfit (gcf, "figures/myfig.tex")
## @end group
## @end example
## will create the "figures" directory in the current directory if it
## doesn't exist and then write "myfig.tex"  and its data files in "figures" 
## 
## 
## If no extension is provided, a
## ".tex" extension will be appended.
## The @var{opt} string argument may be the following:
## @table @asis
## @item "compile"
## @var{val} ("eps" | "pdf"): use system calls to compile/convert the
## figure. Required programs are "pdflatex" for pdf output, or "latex" and
## "ps2eps" for eps output. 
##
## @item "size"
## @var{val} is a 2 element vector containing width and height of the
## figure in centimeters. By default (value []) the dimensions are retrieved
## from "defaultfigurepaperposition" property of the root figure.
##
## @item "keepmaster"
## @var{val} (true | @{false@}): use logical true to not delete
## the master file after compilation. This option has no effect
## without "compile" option
## 
## @end table
## @end deftypefn

## Author: pdiribarne <pdiribarne@macbook-pro-de-pantxo-diribarne.local>
## Created: 2012-02-20

function pgfit (hdl, filepath, varargin)
  nargs = numel (varargin);

  ## argument checking
  if (!ishandle (hdl)) 
    error ("pgfit: first argument should be a graphic handle.")
  elseif (!ischar (filepath))
    error ("pgfit: second argument should be a string.")
  elseif (!isempty (varargin) &&
          !all (cellfun (@ischar, varargin(1:2:end))))
    error ("pgfit: expect strings as \"opt\" argument")
  elseif (floor (numel (varargin)/2) != numel (varargin)/2)
    error ("pgfit: expect OPT/VAL pairs.")
  endif
  more off
  
  ## check path and create needed directories
  [directory, name, ext, ver] = fileparts (filepath);
  if (! isempty (ext))
	if (! isempty (directory))
	  filepath = [directory "/" name];
	else
	  filepath = name;
	endif
  endif
  more off
  dirlist = strsplit (filepath, "/")(1:end-1);
  nitem = numel (dirlist);
  if (nitem > 0)
    idx = 1;
    while idx <= nitem
      if (!any (strcmp (dirlist(idx), {".", "..", ""})))
        pth = fullfile (dirlist(1:idx){:});
        if (exist (pth, "dir") != 7)
          [status, msg] = mkdir (fullfile (dirlist(1:idx){:}));
          if (!status)
            error ("pgfit: %s.", msg)
          endif
        endif
      endif
      idx ++;
    endwhile
  endif
  
  ## Options
  [reg, compile, sizev, keepmaster] = parseparams (varargin,
                                                   "compile", "",
                                                   "size", [],
                                                   "keepmaster", false);
  if (!ischar (compile) ||
      !any (strcmp (compile, {"pdf", "eps", ""}))) 
    error ("pgfit: expect \"pdf\" or \"eps\" for \"compile\" option.");
  endif
  if (!islogical (keepmaster)) 
    error ("pgfit: expect logical for \"keepmaster\" option.");
  endif
  if (!isnumeric (sizev))
    error ("pgfit: numeric vector for \"size\" option.")
  elseif (numel (sizev) != 2 && !isempty (sizev))
    error ("pgfit: expect two elements vector for \"size\" option.")
  elseif (any (sizev < 0))
    error ("pgfit: expect positive width and height")
  elseif (any (sizev > 100))
    warning ("pgfit: %3.2f cm is a huge width")
  elseif (isempty (sizev))
    width = [];
    height = [];
  else
    width = sizev(1);
    height = sizev(2);
  endif
 
  ## write figure file
  pgf_write_figurefile (hdl, filepath, width, height);
  
  if (!isempty (compile))
    pgfcompile (filepath, compile, "lualatex", keepmaster);
  endif
endfunction


function pgf_write_figurefile (h, filepath, width = [], height = [])
  showhidden = "off";
  if (! ishandle (h))
    error ("pgf_write_figurefile: expect handle as first argument")
  elseif (! strcmp (get (h, "type"), "figure"))
    error ("pgf_write_figurefile: expect figure handle")
  elseif (! ischar (filepath))
    error ("pgf_write_figurefile: expect string as second argument")
  elseif (strcmp (get (0, "showhiddenhandles"), "on"))
    set (0, "showhiddenhandles", showhidden)
    showhidden = get (0, "showhiddenhandles");
  endif
  
  ## figure : size, colormap
  units =  get (h, "paperunits");
  if (strcmp (units, "centimeters"))
    fig = get (h);
  else
    set (h, "paperunits", "centimeters");
    fig = get (h);
    set (h, "paperunits", units);
  endif
  pos = fig.paperposition;
  if (isempty (width))
    figwidth = pos(3);
  else
    figwidth = width;
  endif
  if (isempty (height))
    figheight = pos(4);
  else
    figheight = height;
  endif
  cmap = get (h, "colormap");
  clist {1}= cmap(1,:);
  clist {2}= cmap(floor (end *1/3),:);
  clist {3}= cmap(floor (end *2/3),:);
  clist {4}= cmap(end,:);

  texstr = "";

  ## axes
  axs = get (h, "children");
  lg = findobj (h, "tag", "legend");
  tst = arrayfun (@(x) any (x == lg), axs);
  axs(tst) = [];
  cb = findobj (h, "tag", "colorbar");
  tst = arrayfun (@(x) any (x == cb), axs);
  axs(tst) = [];
  axs = axs(end:-1:1);
  
  nbax = numel (axs);
  for ll = 1:nbax
    ax = get (axs(ll));
    ax.nb = ll;
    ax.path = filepath;
    ## colorbar
    tst = [];
    if (numel (cb) > 1)
      tst = cellfun (@(x) any (x == axs(ll)),
                     get (cb, "axes"));
    elseif (numel (cb) > 0)
      tst = (axs(ll) ==  get (cb, "axes"));
    else
      tst = false;
    endif
    ax.cb = cb(tst);
    ax.clims = get (cb, "ylim");
    
    ## legend
    tst = [];
    if (numel (lg) > 1)
      tst = cellfun (@(x) any (x.handle == axs(ll)),
                     get (lg, "userdata"));
    elseif (numel (lg) > 0)
      tst = (axs(ll) ==  get (lg, "userdata").handle(1));
    else
      tst = false;
    endif
    ax.lg = lg(tst);

    ## position
    ax.position(1:2:end) *= figwidth;
    ax.position(2:2:end) *= figheight;

    ## colormap
    ax.clist = clist;
    ax.cmap = colormap ();
    
    ##content
    kids = ax.children;
    cnt = "";
    nkids = numel (kids);
    nline = 0;
    linedata = [];
    nim = 0;
    imdata = [];
    nsurf = 0;
    surfdata = [];
    nscat = 0;
    scatdata = [];
    ncont = 0;
    contdata = [];
    while (nkids)
      ch = get (kids(nkids));
      typ = ch.type;
      if (strcmp (ch.visible, "on"))
        switch ch.type
          case "line"
            nline ++;
            [str, linedata] = printline (ax, ch, nline, linedata);
            cnt = strcat (cnt, "\n", str);
            nkids --;
          case "image"
            nim ++;
            [str, imdata] = printimage (ax, ch, nim, imdata);
            cnt = strcat (cnt, "\n", str);
            nkids --;
          case "text"
            cnt = strcat (cnt, "\n", printtext (ax, ch));
            nkids --;
          case "surface"
            nsurf ++;
            [str, surfdata] = printsurf (ax, ch, nsurf, surfdata);
            cnt = strcat (cnt, "\n", str);
            nkids --;
          case "hggroup"
            typ = gethgtype (ch);
            switch typ
              case "errorbar"
                nline ++;
                [str, linedata] = printerr (ax, ch, nline, linedata);
                cnt = strcat (cnt, "\n", str);
              case "contour"
                ncont ++;
                [str, contdata] = printcontour (ax, ch, ncont, contdata);
                cnt = strcat (cnt, "\n", str);
              case "scatter"
                nscat ++;
                [str, scatdata] = printscatter (ax, ch, nscat, scatdata);
                cnt = strcat (cnt, "\n", str);
              case "bar"
                nline ++;
                ax.barwidth = "10pt";
                [str, linedata] = printbar (ax, ch, nline, linedata);
                cnt = strcat (cnt, "\n", str);
              case {"", "quiver", "scatter3"}
                warning ("pgfit: unrecognized or unimplemented hggroup, some output where skipped!")
            endswitch
            nkids --;
          otherwise
            warning ("pgfit: %s not implemented, some output where skipped!", ch.type)
            nkids --;
        endswitch
      endif
    endwhile
    texstr = strcat (texstr, "\n", printaxes (ax, cnt));
    
    ## write data files
    writedata (ax, linedata, surfdata, imdata, contdata, scatdata);
  endfor

  ## write tex file
  tex_id = fopen ([filepath ".tex"], "w+");
  template = "\\begin{tikzpicture} \n%s\n\\end{tikzpicture} \n";
  fprintf (tex_id, template, texstr);
  fclose (tex_id);
endfunction

function out = logical_str (logical_num)
  if logical_num
    out = "true";
  else
    out = "false";
  endif
endfunction

function strout = curlstr (str)
  strout = ["{", str, "}"];
endfunction


function strout = printtext (ax, ch)
  pos = ch.position;
  rot = ch.rotation;
  str = curlstr (ch.string);
  
  anchor = "";
  temp = ch.verticalalignment;
  if (any (strcmp (temp, {"cap", "top"})))
    temp2 = "north ";
  elseif (any (strcmp (temp, {"baseline", "bottom"})))
    temp2 = "south ";
  else
    temp2 = "";
  endif
  temp = ch.horizontalalignment;
  if strcmp(temp, "left")
    anchor = [temp2 "west"];
  elseif strcmp (temp, "right") 
    anchor = [temp2 "east"];
  else
    anchor = [temp2 ""];
  endif
  if (isempty(anchor))
    anchor="center";
  endif
  template = "\\node[rotate=%f,anchor=%s] at (axis cs:%3.2e,%3.2e){%s};\n";
  strout = sprintf (template, rot, anchor, pos(1), pos(2), str);
endfunction



function [strout, linedata] = printline (ax, li, nb, linedata, err = 0)
  if (!err)
    linedata(nb).X = li.xdata;
    linedata(nb).Y = li.ydata;
    linedata(nb).xldata = [];
    linedata(nb).xudata = [];
    linedata(nb).ldata = [];
    linedata(nb).udata = [];
    table_name = [ax.path,"_line", num2str(ax.nb),".table"];
    linedata(nb).filepath = table_name; 
  else
    linedata(nb).X = li.xdata;
    linedata(nb).Y = li.ydata;
    linedata(nb).xldata = li.xldata;
    linedata(nb).xudata = li.xudata;
    linedata(nb).ldata = li.ldata;
    linedata(nb).udata = li.udata;
    linedata(nb).issymx = [];
    linedata(nb).issymy = [];
    table_name = [ax.path,"_line", num2str(ax.nb),".table"];
    linedata(nb).filepath = table_name;
  endif
  li.nb = nb;

  strout = "";
  # color/mark translation
  [plotopts, defs] = pgf_defopt_plot (li); 
    
  # data;
  tableopts = sprintf ("x=%s, y=%s", ...
                       ["X",num2str(nb)], ["Y",num2str(nb)]);
    
  # color definitions
  strout = sprintf ("%s%s", strout, defs); 

  # legend
  if (! isempty (ax.lg));
    if (! isempty (li.displayname))
      strout = sprintf ("%s\\addlegendentry{%s};\n",
                        strout, li.displayname);
    else
      plotopts = ["forget plot, ", plotopts];
    endif
  endif
    
  if (err)  
    form = li.format;
    ## Fix for unimplemented asymetric errorbars
    if (strfind ("forget plot,", plotopts))
      auxplotopts = sprintf ("%s,\n error bars/.cd", plotopts);
    else
      auxplotopts = sprintf ("%s,\n forget plot, error bars/.cd", plotopts);
    endif
    auxtableopts = tableopts;

    plotopts = sprintf ("%s,\n error bars/.cd", plotopts);
    if (strcmp (form, "xerr") || strcmp (form, "xyerr"))
      linedata(nb).issymx = all (li.xldata == li.xudata);
      if (linedata(nb).issymx)
        plotopts = sprintf ("%s, x dir=both,x explicit", plotopts);
        tableopts = sprintf ("%s, x error=%s", ...
                             tableopts, ["dX",num2str(nb)]);
      else
          plotopts = sprintf ("%s, x dir=minus,x explicit", plotopts);
          auxplotopts = sprintf ("%s, x dir=plus,x explicit", auxplotopts);
          tableopts = sprintf ("%s, x error=%s", tableopts,
                               ["dXL", num2str(nb)]);
          auxtableopts = sprintf ("%s, x error=%s", auxtableopts,
                                  ["dXU", num2str(nb)]);
      endif
    endif
    if (strcmp (form, "yerr") || strcmp (form, "xyerr"))
      linedata(nb).issymy = all (li.ldata == li.udata);
      if (linedata(nb).issymy)
        plotopts = sprintf ("%s, y dir=both,y explicit", plotopts);
        tableopts = sprintf ("%s, y error=%s", ...
                             tableopts, ["dY",num2str(nb)]);
      else
          plotopts = sprintf ("%s, y dir=minus,y explicit", plotopts);
          auxplotopts = sprintf ("%s, y dir=plus,y explicit", auxplotopts);
          tableopts = sprintf ("%s, y error=%s", tableopts,
                               ["dYL", num2str(nb)]);
          auxtableopts = sprintf ("%s, y error=%s", auxtableopts,
                                  ["dYU", num2str(nb)]);
      endif
    endif
  endif
  strout = sprintf ("%s\\addplot[%s] table[%s]{%s};\n", ...
                    strout, plotopts, tableopts, table_name);
  if (err && !all ([linedata(nb).issymx linedata(nb).issymy]))
    strout = sprintf ("%s\n%s\n", ...
                      strout, "%Fix for unimplemented asymetric errorbars");
    strout = sprintf ("%s\\addplot[%s] table[%s]{%s};\n", ...
                      strout, auxplotopts, auxtableopts, table_name);
  endif
  
endfunction

function [strout, errdata] = printerr (ax, err, nb, errdata)
  li = get (err.children(2));
  li.ldata = [];
  li.udata = [];
  li.xldata = [];
  li.xudata = [];
  li.format = "";
 
  if (isfield (err, "ldata") && isfield(err, "udata"))
    li.ldata = err.ldata;
    li.udata = err.udata;
  endif
  
  if (isfield (err, "xldata") && isfield (err, "xudata"))
    li.xldata = err.xldata;
    li.xudata = err.xudata;
  endif
    
  form = err.format;
  if (strcmp (form, "xerr") || strcmp (form, "box"))
    li.format = "xerr";
  elseif (strcmp (form, "yerr") || strcmp (form, "boxy"))
    li.format = "yerr";
  else
    li.format = "xyerr";
  endif

  ## displayname
  if (!isempty (ax.lg))
    lgkids = findobj (get (ax.lg, "children"), "type", "text");
    nkids = numel (lgkids);
    while nkids
      if (get (lgkids(nkids), "userdata") == err.children(1))
        li.displayname = get (lgkids(nkids), "string");
        nkids = 0;
      else
        nkids --;
      endif
    endwhile
  endif
  [strout, errdata] = printline (ax, li, nb, errdata, 1);
endfunction

function [strout, surfdata] = printsurf (ax, su, nb, surfdata, err = 0)
  table_path = sprintf ("%s_surf%d_%d.table", ax.path, ax.nb, nb);
  x = su.xdata;
  y = su.ydata;
  z = su.zdata;
  
  #data
  if (numel (z) > (60*60))
    error ("pgfit : too large number of points in surface data")  
  elseif (isvector (x) && isvector (y)) 
    [x, y] = meshgrid (x, y);
  endif
  surfdata(nb).X = x;
  surfdata(nb).Y = y;
  surfdata(nb).Z = z;
  surfdata(nb).filepath = table_path;
  
  # View, Shading, colorbar properties
  shading = su.facecolor;
  if (!isempty (ax.cb))
    clist = ax.clist;
  endif
  strout = sprintf ("\\addplot3[surf, shader=%s] file {%s};  \n", ...
                    shading, table_path);
endfunction

function [strout, imdata] = printimage (ax, im, nb, imdata)
  impath = sprintf ("%s_im%d_%d.png", ax.path, ax.nb, nb);
  if (strcmp (im.cdatamapping, "scaled"))
    if (all (ax.cmap(:,1) == ax.cmap(:,2)))
      imdata(nb).cdata = im.cdata;
    else
      imdata(nb).cdata = gray2ind (im.cdata, rows (ax.cmap));
      imdata(nb).cdata = ind2rgb (imdata(nb).cdata);
    endif
  else
    imdata(nb).cdata = im.cdata;
  endif
  imdata(nb).filepath = impath;

  template = "\\addplot graphics[xmin=%3.2f,xmax=%3.2f,ymin=%3.2f,ymax=%3.2f] {%s};\n";
  strout = sprintf (template , ax.xlim(1), ax.xlim(2),
                    ax.ylim(1), ax.ylim(2), impath);
endfunction

function [strout, contdata] = printcontour (ax, ct, ncont, contdata);
  contdata(ncont).contourmatrix = ct.contourmatrix;
  contdata(ncont).filepath = [ax.path, "_contour", num2str(ax.nb),".table"];
  template = "\\addplot+[mark=,contour prepared,contour prepared format=matlab] file {%s};\n";
  strout = sprintf (template, contdata(ncont).filepath);
endfunction

function [strout, linedata] = printbar (ax, ba, nbar, linedata);
  linedata(nbar).X = ba.xdata;
  linedata(nbar).Y = ba.ydata;
  linedata(nbar).ldata = [];
  linedata(nbar).udata = [];
  linedata(nbar).xldata = [];
  linedata(nbar).xudata = [];
  linedata(nbar).cdata = get (ba.children, "facecolor");
  linedata(nbar).filepath =  sprintf ("%s_line%d.table",
                                      ax.path, ax.nb);
  
  # data
  if (strcmp (linedata(nbar).cdata, "flat"))
    template = "\\addplot+[mark=] table [x=%s,y=%s]{%s};\n";
    strout = sprintf (template, ["X", num2str(nbar)],
                      ["Y", num2str(nbar)], linedata(nbar).filepath);
  else
    [col, def] = octcol2pgf (linedata(nbar).cdata,
                             ["barcol", num2str(nbar)]);
    if (isempty (def))
      template = "\\addplot+[fill=%s, draw=%s] table [x=%s,y=%s]{%s};\n";
      strout = sprintf (template, col, col, ["X", num2str(nbar)],
                        ["Y", num2str(nbar)], linedata(nbar).filepath);
    else
      template = [def "\n\\addplot+[fill=%s, draw=%s] table [x=%s,y=%s]{%s};\n"];
      strout = sprintf (template, ["barcol", num2str(nbar)],
                      ["barcol", num2str(nbar)], ["X", num2str(nbar)],
                      ["Y", num2str(nbar)], linedata(nbar).filepath);
    endif
  endif
  # legend
  if (! isempty (ax.lg));
     dispname = get (ba.children, "displayname");
    if (! isempty (dispname))
      strout = sprintf ("%s\\addlegendentry{%s};\n",
                        strout, dispname);
    endif
  endif
endfunction

function str = printaxes (ax, cnt)
  defs = "";
  ncol = 0;
  show = strcmp (ax.visible, "on");
  # axes type
  xtype = strcmp (ax.xscale, "log");
  ytype = strcmp (ax.yscale, "log");
  if (xtype+ytype) == 0
    typ = "axis";
  elseif (xtype+ytype) == 2
    typ = "loglogaxis";
  elseif xtype
    typ = "semilogxaxis";
  else
    typ = "semilogyaxis";
  end


  str = "";
  #size and location of the axes
  loc = ax.position;
  str = sprintf ("%sscale only axis,\n", str);
  str = sprintf ("%sat={(%fcm,%fcm)},\n", ...
                 str, loc(1), loc(2));
  str = sprintf ("%swidth=%3.2fcm, height=%3.2fcm,\n", ...
                 str, loc(3), loc(4));
  

  ## plotyy
  if (isfield (ax, "__plotyy_axes__"))
    str = sprintf ("%saxis y line*=%s,\n", str, ax.yaxislocation);
  endif

  ## bar plot
  if (isfield (ax, "barwidth"))
    str = sprintf ("%sybar,\n", str);
    str = sprintf ("%sbar width=%s,\n", str, ax.barwidth);
  endif

  ## color
  if (!all (ax.ycolor == [0 0 0]))
     [col, tmp] = octcol2pgf (ax.ycolor, sprintf ("axcol%d%d", ax.nb, ncol));
     if (!isempty (tmp))
       defs = sprintf ("%s\n%s", defs, tmp);
       str = sprintf ("%sy axis line style=axcol%d%d,\n",
                      str, ax.nb, ncol);
       str = sprintf ("%sy tick label style=axcol%d%d,\n",
                      str, ax.nb, ncol);
       str = sprintf ("%sy label style=axcol%d%d,\n",
                      str, ax.nb, ncol);
       
       ncol ++;
     else
       str = sprintf ("%sy axis line style=%s,\n", str, col);
       str = sprintf ("%sy tick label style=%s,\n", str, col);
       str = sprintf ("%sy label style=%s,\n", str, col);
     endif
  endif
  
  ## box
  if (strcmp (ax.box, "off") && all (ax.view == [0 90]))
    %str = sprintf ("%saxis x line*=%s,\n", str, ax.xaxislocation);
  endif
  
  #colorbar : defined arbitrarily with 4 points only
  if (! isempty (ax.cb))
    clist = ax.clist;
    str = sprintf ("%scolormap={mycmap}{rgb(0cm)=(%f,%f,%f);rgb(1cm)=(%f,%f,%f);rgb(2cm)=(%f,%f,%f);rgb(3cm)=(%f,%f,%f)},\n", str, ...
                   clist{1}(1), clist{1}(2), clist{1}(3), ...
                   clist{2}(1), clist{2}(2), clist{2}(3), ...
                   clist{3}(1), clist{3}(2), clist{3}(3), ...
                   clist{4}(1), clist{4}(2), clist{4}(3));
    str = sprintf ("%spoint meta min={%f},\n", str, ax.clims(1));
    str = sprintf ("%spoint meta max={%f},\n", str, ax.clims(2));

    str = sprintf ("%scolorbar,\n", str);
  endif

  ## legend
  if (! isempty (ax.lg))
    loc = get (ax.lg, "location");
    orient = get (ax.lg, "orientation");
    nitem = numel (get (ax.lg, "string"));
    box = get (ax.lg, "box");
    tmp = "";
    style = "";
    if (!isempty(idx = strfind (loc, "outside")))
      tmp = [tmp " outer"];
    endif
    if (!isempty(idx = strfind (loc, "north")))
      tmp = [tmp " north"];
    elseif (!isempty(idx = strfind (loc, "south")))
      tmp = [tmp " south"];
    endif
    if (!isempty(idx = strfind (loc, "east")))
      tmp = [tmp " east"];
    elseif (!isempty(idx = strfind (loc, "west")))
      tmp = [tmp " west"];
    endif

    template = "at={(%f,%f)}, anchor=%s";
    switch (strtrim (tmp))
      case {"south west", "south east", "north west", ...
            "north east", "outer north east"}
        str = sprintf ("%slegend pos=%s,\n", str, tmp);
      case "north"
        style = sprintf(template, 0.5, 0.98, "north");
      case "east"
        style = sprintf(template, 0.98, 0.5, "east");
      case "south"
        style = sprintf(template, 0.5, 0.02, "south");
      case "west"
        style = sprintf(template, 0.02, 0.5, "west");
      case "outer north"
        style = sprintf(template, 0.5, 1.02, "south");
      case "outer east"
        style = sprintf(template, 1.02, 0.5, "west");
      case "outer south east"
        style = sprintf(template, 1.02, 0, "south west");
      case "outer south"
        style = sprintf(template, 0.5, -0.08, "north");
      case "outer west"
        style = sprintf(template, -0.08, 0.5, "east");
      case "outer south west"
        style = sprintf(template, -0.08, 0, "south east");
      case "outer north west"
        style = sprintf(template, -0.08, 1, "north east");
    endswitch
    if (strcmp (orient, "horizontal"))
      str = sprintf ("%slegend columns=%d,\n", str, nitem);
    endif
    if (strcmp (box, "off"))
      if (!isempty (style))
        str = sprintf ("%slegend style={draw=none, %s},\n", str, style);
      else
        str = sprintf ("%slegend style={draw=none},\n", str);
      endif
    elseif (!isempty (style))
      str = sprintf ("%slegend style={%s},\n", str, style);
    endif
  endif
  
  #labels
  if (show)
    labels = {"title", "xlabel", "ylabel", "zlabel"};
    for ii = 1:numel (labels)
      str = sprintf ("%s%s=%s,\n", str, labels{ii},
                     curlstr (get (ax.(labels{ii}), "string")));
    endfor
  endif
  
  #limits
  str = sprintf ("%sxmin=%f,\n", str, ax.xlim(1));
  str = sprintf ("%sxmax=%f,\n", str, ax.xlim(2));
  str = sprintf ("%symin=%f,\n", str, ax.ylim(1));
  str = sprintf ("%symax=%f,\n", str, ax.ylim(2));
  if (!all (ax.view == [0 90]))
    str = sprintf ("%szmin=%f,\n", str, ax.zlim(1));
    str = sprintf ("%szmax=%f,\n", str, ax.zlim(2));
  endif
  
  #grid
  str = sprintf ("%sxminorgrids=%s,\n", str,
                 logical_str (strcmp (ax.xminorgrid, "on")));
  str = sprintf ("%syminorgrids=%s,\n", str,
                 logical_str (strcmp (ax.yminorgrid, "on")));
  str = sprintf ("%szminorgrids=%s,\n", str,
                 logical_str (strcmp (ax.zminorgrid, "on")));
  str = sprintf ("%sxmajorgrids=%s,\n", str,
                 logical_str (strcmp (ax.xgrid, "on")));
  str = sprintf ("%symajorgrids=%s,\n", str,
                 logical_str (strcmp (ax.ygrid, "on")));
  str = sprintf ("%szmajorgrids=%s,\n", str,
                 logical_str (strcmp (ax.zgrid, "on")));


  ## ticks
  if (!show)
    str = sprintf ("%sxtick=\\empty,\nytick=\\empty,\n", str);
  else
    if (isempty (ax.xticklabel))
      str = sprintf ("%sxtick=\\empty,\n", str);
    else
      ticks = ax.xtick;
      labels = ax.xticklabel;
      tmp = "";
      tmp2 = "";
      for ii = 1:numel (ticks)
        tmp = append_opt (tmp, num2str (ticks(ii)));
        if (isempty (labels{ii}))
          labels{ii} = " ";
        endif
        tmp2 = append_opt (tmp2, labels{ii});
      endfor
      str = sprintf ("%sxtick={%s},\n", str, tmp);
      str = sprintf ("%sxticklabels={%s},\n", str, tmp2);
    endif
    if (isempty (ax.yticklabel))
      str = sprintf ("%sytick=\\empty,\n", str);
    else
      ticks = ax.ytick;
      labels = ax.yticklabel;
      tmp = "";
      tmp2 = "";
      for ii = 1:numel (ticks)
        tmp = append_opt (tmp, num2str (ticks(ii)));
        if (isempty (labels{ii}))
          labels{ii} = " ";
        endif
        tmp2 = append_opt (tmp2, labels{ii});
      endfor
      str = sprintf ("%sytick={%s},\n", str, tmp);
      str = sprintf ("%syticklabels={%s},\n", str, tmp2);
    endif
  endif
  
  ## view
  if (!all (ax.view == [0 90]))
    str = sprintf ("%sview={%f}{%f},\n", str, ...
                   ax.view(1), ax.view(2)); 
  endif
  
  str = sprintf ("%senlargelimits=false,\n", str);
  if (!isempty (defs))
     str = sprintf ("%s\n\\begin{%s}[%s]\n%s\n\\end{%s}",
                    defs, typ, str, cnt, typ);
  else
    str = sprintf ("\\begin{%s}[%s]\n%s\n\\end{%s}",
                   typ, str, cnt, typ);
  endif
endfunction

function [lineopt, defs] = pgf_defopt_plot (li)
  mark = li.marker;
  marksize = li.markersize;
  linestyle = li.linestyle;
  linewidth = li.linewidth;
  linecolor = li.color;
  if (ischar(li.markerfacecolor))
    if strcmp(li.markerfacecolor, "none")
      has_face = 0;
      markface = [1,1,1];
    endif
  else
    has_face = 1;
    markface = li.markerfacecolor;
  endif

  if (ischar(li.markeredgecolor))
    if strcmp(li.markeredgecolor, "auto")
      has_edge = 1;
      markedge = linecolor;
    elseif (strcmp(li.markeredgecolor, "none"))
      has_edge = 0;
      markedge = [1,1,1]
    endif
  else
    has_edge = 1;
    markedge = li.markeredgecolor;
  endif
  nb = li.nb;
  sizefactor = 0.78/6;
  markopt=["scale=", num2str(sizefactor*marksize)];
  lineopt="";
  defs = "";
  ##lines
  if ! strcmp(linestyle, "none")
    if strcmp (linestyle, ":")
      pgf_line = "dotted";
    elseif strcmp (linestyle, "-")
      pgf_line = "solid";
    elseif strcmp (linestyle, "--")
      pgf_line = "dashed";
    elseif strcmp (linestyle, "-.")
      pgf_line = "dashdotted";
    endif
    lineopt = append_opt(lineopt, pgf_line);
  else
    pgf_line = "";
    lineopt = append_opt (lineopt, "only marks")
  endif
  lineopt = append_opt (lineopt, sprintf ("line width=%.1fpt", linewidth));
  ##marks
  if ! has_face
    if strcmp (mark, ".")
      pgf_mark = "";
    elseif strcmp (mark, "none")
      pgf_mark = "";
    elseif strcmp (mark, "*")
      pgf_mark = "asterisk";
    elseif strcmp (mark, "o")
      pgf_mark = "o";
    elseif strcmp (mark, "x")
      pgf_mark = "x";
    elseif strcmp (mark, "+")
      pgf_mark = "+";
    elseif strcmp (mark, "s")
      pgf_mark = "square";
    elseif strcmp (mark, "d")
      pgf_mark = "diamond";
    elseif strcmp (mark, "h")
      pgf_mark = "asterisk";
    elseif strcmp (mark, "^")
      pgf_mark = "triangle";
    elseif strcmp (mark, "<")
      pgf_mark = "triangle";
      markopt = append_opt (markopt, "rotate=90");
    elseif strcmp (mark, "v")
      pgf_mark = "triangle";
      markopt = append_opt (markopt, "rotate=180");
    elseif strcmp (mark, ">")
      pgf_mark = "triangle";
      markopt = append_opt (markopt, "rotate=-90");
    elseif strcmp (mark, "p")
      pgf_mark = "+";
    else
      pgf_mark = "none";
    endif
  else
    if strcmp (mark, ".")
      pgf_mark = "";
    elseif strcmp (mark, "none")
      pgf_mark = "";
    elseif strcmp (mark, "*")
      pgf_mark = "asterisk";
    elseif strcmp (mark, "o")
      pgf_mark = "*";
    elseif strcmp (mark, "x")
      pgf_mark = "x";
    elseif strcmp (mark, "+")
      pgf_mark = "+";
    elseif strcmp (mark, "s")
      pgf_mark = "square*";
    elseif strcmp (mark, "d")
      pgf_mark = "diamond*";
    elseif strcmp (mark, "h")
      pgf_mark = "asterisk";
    elseif strcmp (mark, "^")
      pgf_mark = "triangle*";
    elseif strcmp (mark, "<")
      pgf_mark = "triangle*";
      markopt = append_opt (markopt, "rotate=90");
    elseif strcmp (mark, "v")
      pgf_mark = "triangle*";
      markopt = append_opt (markopt, "rotate=180");
    elseif strcmp (mark, ">")
      pgf_mark = "triangle*";
      markopt = append_opt (markopt, "rotate=-90");
    elseif strcmp (mark, "p")
      pgf_mark = "+";
    else
      pgf_mark = "none";
    endif
    
  endif
  ##colors
  if (! isempty(pgf_line))
    [col, str] = octcol2pgf (linecolor, ["linecol", num2str(nb)]);
    if (!isempty (str))
      defs = [defs, str];
      lineopt = append_opt (lineopt, ["color=linecol", num2str(nb)]);
    else
      lineopt = append_opt (lineopt, ["color=", col]);
    endif
  endif
  
  if (!isempty (pgf_mark))
    [col, str] = octcol2pgf (markedge, ["edgecol", num2str(nb)]);
    if (!isempty (str))
      defs = [defs, str];
      markopt = append_opt (markopt, ["solid, draw=edgecol", num2str(nb)]);
    else
      markopt = append_opt (markopt, ["solid, draw=", col]);
    endif
    if (has_face)
      [col, str] = octcol2pgf (markface, ["facecol", num2str(nb)]);
      if (!isempty (str))
        defs = [defs, str];
        markopt = append_opt (markopt, ["fill=facecol", num2str(nb)]);
      else
        markopt = append_opt (markopt, ["fill=", col]);
      endif
    endif
    lineopt = append_opt (lineopt, ["mark=", pgf_mark]);
    lineopt = append_opt (lineopt, ["mark options={", markopt, "}"]);
  endif
endfunction

function strout = append_opt (strin, addon)
  if !isempty(strin)
    strout = [strin, ", ", addon];
  else
    strout = addon;
  endif
endfunction

function writedata (ax, linedata, surfdata, imdata, contdata, scatdata)
  ## lines
  if (!isempty (linedata))
    args = {};
    col = 1;
    line_nb = length (linedata);
    for nb = 1:line_nb
      args{col++} = ["X",num2str(nb)];
      args{col++} = linedata(nb).X;
      args{col++} = ["Y",num2str(nb)];
      args{col++} = linedata(nb).Y;
      if (!isempty (linedata(nb).xldata))
         if (linedata(nb).issymx)
           args{col++} = ["dX",num2str(nb)];
           args{col++} = (linedata(nb).xldata + linedata(nb).xudata)/2;
         else
           args{col++} = ["dXL",num2str(nb)];
           args{col++} = linedata(nb).xldata;
           args{col++} = ["dXU",num2str(nb)];
           args{col++} = linedata(nb).xudata;
         endif
      endif
      if (!isempty (linedata(nb).ldata))
         if (linedata(nb).issymy)
           args{col++} = ["dY",num2str(nb)];
           args{col++} = (linedata(nb).ldata + linedata(nb).udata)/2;
         else
           args{col++} = ["dYL",num2str(nb)];
           args{col++} = linedata(nb).ldata;
           args{col++} = ["dYU",num2str(nb)];
           args{col++} = linedata(nb).udata;
         endif
        
      endif
    endfor
    table_name = linedata(1).filepath;
    pgf_write_table (table_name , args);
  endif
  
  ## surfaces
  if (!isempty (surfdata))
    for ll = 1:numel (surfdata)
		surfdata(ll).filepath
      table_id = fopen (unix2native (surfdata(ll).filepath), "w"); 
      for ii = 1: length (surfdata(ll).X(:,1))
        for jj = 1: length (surfdata(ll).X(1,:))
          fprintf (table_id, "%3.2e %3.2e %3.2e \n",
                   surfdata(ll).X(ii,jj),
                   surfdata(ll).Y(ii,jj),
                   surfdata(ll).Z(ii,jj));
        endfor
        fprintf (table_id, "\n");
      endfor
    endfor
    fclose (table_id);
  endif

  ## images
  if (!isempty (imdata))
    for ll = 1:numel (imdata)
      imwrite (imdata(ll).cdata, imdata(ll).filepath, ax.cmap);
    endfor
  endif

  ## contours
  if (!isempty (contdata))
    cont_nb = length (contdata);
    for nb = 1:cont_nb
      contourmatrix =  (contdata(nb).contourmatrix)';
      save ("-ascii", contdata(nb).filepath, "contourmatrix");
    endfor
  endif

  ## scatter
  if (!isempty (scatdata))
    scat_nb = length (scatdata);
    args = {};
    col = 1;
    for nb = 1:scat_nb
      args{col++} = ["X",num2str(nb)];
      args{col++} = scatdata(nb).xdata;
      args{col++} = ["Y",num2str(nb)];
      args{col++} = scatdata(nb).ydata;
      args{col++} = ["COL",num2str(nb)];
      args{col++} = scatdata(nb).cdata;
      pgf_write_table (scatdata(nb).filepath , args);
    endfor
  endif

endfunction

function typ = gethgtype (hg)
  if (isfield (hg, "contourmatrix"))
     typ = "contour";
  elseif (isfield (hg, "format"))
    typ = "errorbar";
  elseif (isfield (hg, "vdata"))
    typ = "quiver";
  elseif (isfield (hg, "sizedata") && isempty (hg.zdata))
    typ = "scatter";
  elseif (isfield (hg, "sizedata"))
    typ = "scatter3";
  elseif (isfield (hg, "bargroup"))
    typ = "bar";
  else
    typ = "";
  endif
endfunction

function [col, def] = octcol2pgf (incol, name);
  ii = 0;
  def = "";
  col = "";
  
  ii ++;
  tmp.octname = "r";
  tmp.rgb = [1 0 0];
  tmp.pgfname = "red";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "g";
  tmp.rgb = [0 1 0];
  tmp.pgfname = "green";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "b";
  tmp.rgb = [0 0 1];
  tmp.pgfname = "blue";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "k";
  tmp.rgb = [0 0 0];
  tmp.pgfname = "black";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "w";
  tmp.rgb = [1 1 1];
  tmp.pgfname = "white";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "y";
  tmp.rgb = [1 1 0];
  tmp.pgfname = "yellow";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "c";
  tmp.rgb = [0 1 1];
  tmp.pgfname = "cyan";
  colstruct(ii) = tmp;

  ii ++;
  tmp.octname = "m";
  tmp.rgb = [1 0 1];
  tmp.pgfname = "magenta";
  colstruct(ii) = tmp;

  if (ischar (incol))
    tmp = {colstruct(:).octname};
    tst = find (strcmp (incol, tmp));
    if (tst)
      col = colstruct(tst).pgfname;
    else
      error ("pgfit: unknown colorname %s.", incol)
    endif
  elseif (isnumeric (incol) && numel (incol) == 3)
    tst = find (arrayfun (@(x) all (x.rgb == incol), colstruct));
    if (tst)
      col = colstruct(tst).pgfname;
    else
      col = incol;
      def = sprintf ("\\definecolor{%s}{rgb}{%f,%f,%f};\n", ...
                     name, incol(1), incol(2), incol(3));
    endif
  endif
endfunction             

function pathout = unix2native (pathin)
  [DIR, NAME, EXT, VER] = fileparts (pathin);
  if (!isempty (EXT))
	pathout = fullfile (DIR, [NAME EXT]);
  else
    pathout = fullfile (DIR, NAME);
  endif
endfunction	

%!demo
%! hdl = figure(1234);
%! subplot(2,2,1);
%! plot(rand(1,30),"-ok;random nb;", "markerfacecolor", [1 0 1])
%! subplot(2,2,2); hold on
%! x = 0:0.1:2*pi;
%! y1 = sin (x);
%! y2 = exp (x - 1);
%! ax = plotyy (x, y1, x - 1, y2, @plot, @semilogy); 
%! y3 = sin(x)./x;
%! dy3 = y3/6;
%! dx = 0.2*ones (size (x));
%! errorbar (x, y3, dx, dy3, "#>-or;One legend;*");
%! subplot(2,2,3);
%! sombrero;
%! subplot(2,2,4);
%! contour(rand(10,10), 4)
%! text (5,5,'$\sigma = \sqrt{\frac{\nu}{f_0}}$', ...
%!      'horizontalalignment','center');
%! pgfit (hdl, "myfig", "compile", "pdf");

%!demo
%! hdl = figure(1234); clf; 
%! title ("lines/marks/errorbar demo")
%! hold on; grid on; set(gca, "yminorgrid", "on");
%! y = rand (1,8);
%! l1 = errorbar((1:8), y, y/4, "-g;err;"); 
%! l2 = semilogy(y/10, "-ok;random nb;");
%! set (l2, "markerfacecolor", rand (1,3), "markersize", 22);
%! l3 = semilogy(y/100, "--dr;random nb;");
%! set (l3, "markerfacecolor", [1 0 1], "markersize", 22);
%! l4 = semilogy(y/1000, "-.*b");
%! set (l4, "linewidth", 3, "markersize", 12);
%! pgfit (hdl, "myfig", "size", [10, 8], "compile", "eps")

